from mythx_models.response.group_status import GroupStatusResponse


class GroupCreationResponse(GroupStatusResponse):
    """The API response domain model for the status of a newly created group."""

    pass
